pub struct Frac<T> { /* private fields */ }Expand description
Token that maintains fractional access to some resource.
This allows multiple clients to obtain shared references to some resource
via borrow.
Implementations
Sourceimpl<T> Frac<T>
impl<T> Frac<T>
Sourcepub open spec fn valid(self, id: Loc, frac: real) -> bool
pub open spec fn valid(self, id: Loc, frac: real) -> bool
{
&&& self.id() == id
&&& self.frac() == frac
}Sourcepub proof fn new(tracked v: T) -> tracked result : Self
pub proof fn new(tracked v: T) -> tracked result : Self
result.frac() == 1 as real,result.resource() == v,Create a fractional token that maintains shared access to the input resource.
Sourcepub proof fn agree(tracked &self, tracked other: &Self)
pub proof fn agree(tracked &self, tracked other: &Self)
self.id() == other.id(),ensuresself.resource() == other.resource(),Two tokens agree on values of the underlying resource.
Sourcepub proof fn split_to(tracked &mut self, result_frac: real) -> tracked result : Self
pub proof fn split_to(tracked &mut self, result_frac: real) -> tracked result : Self
(0 as real) < result_frac < old(self).frac(),ensuresself.id() == old(self).id(),result.id() == old(self).id(),self.resource() == old(self).resource(),result.resource() == old(self).resource(),self.frac() == old(self).frac() - result_frac,result.frac() == result_frac,Split one resource into two resources whose quantities sum to the original.
The returned token has quantity result_frac; the new value of the input token has
quantity old(self).frac() - result_frac.
Sourcepub proof fn split(tracked &mut self) -> tracked result : Self
pub proof fn split(tracked &mut self) -> tracked result : Self
self.id() == old(self).id(),result.id() == old(self).id(),self.resource() == old(self).resource(),result.resource() == old(self).resource(),self.frac() == old(self).frac() / 2 as real,result.frac() == old(self).frac() / 2 as real,Split one resource by half into two resources whose quantities sum to the original.
The returned token has quantity n; the new value of the input token has
quantity old(self).frac() - n.
Sourcepub proof fn combine(tracked &mut self, tracked other: Self)
pub proof fn combine(tracked &mut self, tracked other: Self)
old(self).id() == other.id(),ensuresself.id() == old(self).id(),self.resource() == old(self).resource(),self.resource() == other.resource(),self.frac() == old(self).frac() + other.frac(),Combine two tokens, summing their quantities.
Sourcepub proof fn bounded(tracked &self)
pub proof fn bounded(tracked &self)
(0 as real) < self.frac() <= (1 as real),Allowed values for a token’s quantity.
Sourcepub proof fn borrow(tracked &self) -> tracked ret : &T
pub proof fn borrow(tracked &self) -> tracked ret : &T
ret == self.resource(),Obtain shared access to the underlying resource.
Sourcepub proof fn take_resource(tracked self) -> tracked pair : (T, Empty<T>)
pub proof fn take_resource(tracked self) -> tracked pair : (T, Empty<T>)
self.frac() == 1 as real,ensurespair.0 == self.resource(),pair.1.id() == self.id(),Reclaim full ownership of the underlying resource.